﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("CountNumberOfUniqueActions", "")]
  [StorableClass]
  public class CountNumberOfUniqueActions : SingleSuccessorOperator {

    public ILookupParameter<ItemArray<IClassifier>> ClassifiersParameter {
      get { return (ILookupParameter<ItemArray<IClassifier>>)Parameters["Classifiers"]; }
    }
    public ILookupParameter<IClassifierComparer> ClassifierComparerParameter {
      get { return (ILookupParameter<IClassifierComparer>)Parameters["ClassifierComparer"]; }
    }
    public IValueLookupParameter<IItemSet<IAction>> UniqueActionsParameter {
      get { return (IValueLookupParameter<IItemSet<IAction>>)Parameters["UniqueActions"]; }
    }
    public IValueLookupParameter<IntValue> NumberOfUniqueActionsParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["NumberOfUniqueActions"]; }
    }

    [StorableConstructor]
    protected CountNumberOfUniqueActions(bool deserializing) : base(deserializing) { }
    protected CountNumberOfUniqueActions(CountNumberOfUniqueActions original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new CountNumberOfUniqueActions(this, cloner);
    }
    public CountNumberOfUniqueActions()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<IClassifier>("Classifiers"));
      Parameters.Add(new LookupParameter<IClassifierComparer>("ClassifierComparer"));
      Parameters.Add(new ValueLookupParameter<IItemSet<IAction>>("UniqueActions"));
      Parameters.Add(new ValueLookupParameter<IntValue>("NumberOfUniqueActions"));
    }

    public override IOperation Apply() {
      ItemSet<IAction> actions = new ItemSet<IAction>(ClassifierComparerParameter.ActualValue);
      foreach (var classifier in ClassifiersParameter.ActualValue) {
        actions.Add(classifier.Action);
      }
      UniqueActionsParameter.ActualValue = actions;
      NumberOfUniqueActionsParameter.ActualValue = new IntValue(actions.Count);
      return base.Apply();
    }
  }
}
